CSAR = Class:derive("CSAR")
CSAR.Settings = {}
CSAR.Settings.CSARZonesTable = Settings.FARPHollywood_CSAR_Zones -- Tables of CSAR Zone names in ME
CSAR.Settings.AlliedLandingZones = Settings.FARPHollywood_Allied_LZs -- Table of Allied Landing Zones - ZONE:New()
CSAR.Settings.clientTransportsTable = Settings.clientTransportsTable -- Table of avalible client transports GroupNames
CSAR.Settings.TroopHoldZoneName = "FARPHollywood_Hold" -- Name of Zone troops will run to after unboarding
CSAR.Settings.AlliedSoliderGroupName = "Infantry M4 (CSAR)" -- GroupName of Late Activated Solider to serve troop to be rescued
CSAR.Settings.EnemySoliderGroupName = "Insurgent Enemy Solider" -- GroupName of Late Activated Solider to serve as enemy threats
CSAR.Settings.EnemyBTRGroupName = "Insurgent Enemy BTR-80" --GroupName of BTR to spawn for each assignment

CSAR.CSARTroopSpawn = SPAWN:New(CSAR.Settings.AlliedSoliderGroupName)
      :InitKeepUnitNames(true)
      :InitHeading(0,359)

CSAR.CSAREnemyTroopSpawn = SPAWN:New(CSAR.Settings.EnemySoliderGroupName)
      --:InitLimit(4,4)
      :InitHeading(0,359)
      :InitRandomizePosition( true , 600, 350 )
      --:SpawnScheduled(0.5,0)
      
CSAR.CSAREnemyBTR = SPAWN:New(CSAR.Settings.EnemyBTRGroupName)
:InitHeading(0,359)
:InitRandomizePosition( true , 1205, 1200 )


--[[
CSAR:New(PlayerGroupName,CSARZonesTable,AlliedLandingZones,unBoardToUnitName)
PlayerGroupName =  Name of PlayerGroup
CSARZonesTable = Table of avalible zones names made in mission editor { }
AlliedLandingZones = Table of Allied landing Zones 
TroopHoldZone = ZoneTroops will trun to after unboarding safe zone
]]
function CSAR:New(playerGroup,CSARZonesTable,AlliedLandingZones,TroopHoldZoneName)
  local PlayerGroup = playerGroup
  local PlayerGroupName = PlayerGroup:GetName()
  --check if already assigned a CSAR Task
  if CSAR.GetMooseClientStateData(PlayerGroup,"hasAssignment") == true then
    CSAR.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
    MESSAGE:New("You have already been assigned a Task. Please review your briefing from F10 Menu. Complete you current task before selecting a new one.",15):ToGroup(PlayerGroup)
  
  else
    local EnemyGroupSet = SET_GROUP:New()
    local RadioBeaconDuration = 3600 -- max how long a radio beacon to remain active in sec
    local RadioFreq = TheFrat.GetFreq()
    local inCSAR_Zone = false
    local troopPickedUp = false
    local troopUnloaded = false
    local CSARCompleted = false --Is CSAR mission Completed?
    local inAirSpace = false
    local firstTakeoff = false
    local CSAR_Task_ID = TheFrat.RandomNumber()

    --Get Random Zone
    if CSAR.tablelength(CSAR.Settings.CSARZonesTable) >= 1 and CSAR.tablelength(Settings.RadioFreqs) >= 1 then
      
       --Set hasCSAR state to true
       CSAR.SetMooseClientStateData(PlayerGroup,"hasAssignment", true)
    
       local zoneName = CSAR.randomchoice(CSARZonesTable)
       local Zone = ZONE:New(zoneName)
       CSAR.removeTableEntry(CSARZonesTable,zoneName)

       local SpawnVec3 = POINT_VEC3:NewFromVec3( Zone:GetVec3() )
       
       --Place CSAR Troop
       local CSARTroopGroup = CSAR.CSARTroopSpawn:SpawnFromVec2( Zone:GetVec2() )
       local CSARTroopUnit = CSARTroopGroup:GetUnit(1)
       local InfantryCargo = CARGO_GROUP:New( CSARTroopGroup, "Troop", "Troop"..CSAR.RandomVariable(3), 500 )
       local CSARZone = ZONE_UNIT:New(CSARTroopUnit:GetName(), CSARTroopUnit, 2600) --CSAR airspace
       --CSARTroopUnit:SmokeBlue()
       
       --Spawn Random Enemy Units in route to LZ when not in allied zone
      local noEnemyThreatZones = Settings.NoThreatsZones --table
      table.insert(noEnemyThreatZones, Zone)
      --RandomEnemyScheduler:Stop() to stop

       --pop flare/Smoke when near
        if Zone:GetCoordinate():IsDay() == true then
          --day smoke
          local CSARSmoke = SCHEDULER:New( nil,
            function()
              if PlayerGroup:IsCompletelyInZone(CSARZone) and inCSAR_Zone == false then
              inCSAR_Zone = true
              CSAR.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
              PlayerGroup:Message("I hear you near by, popping green smoke. Please hurry!",15)
                --Smoke
                Zone:SmokeZone(SMOKECOLOR.Green,1)
              end
            end, {}, 1,1,0,5400)
        
        else
          --night to flare
          local CSARFlare = SCHEDULER:New( nil,
            function()
              if PlayerGroup:IsCompletelyInZone(CSARZone) and inCSAR_Zone == false then
              inCSAR_Zone = true
              CSAR.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
              PlayerGroup:Message("I hear you near by, popping green flares. Please hurry!",15)
                --pop flare
              SCHEDULER:New( nil,
                function()
                  Zone:FlareZone(FLARECOLOR.Green,1)
                end, {},2,4,0,180)
              end
            end, {}, 1,1,0,5400)
        
        end
       
       
       --Place Enemy Troops & BTR
       local ranNumofTroops = math.random(4,8) --gen random number of troops math.random(3,5)
        for EnemyTroopCount = ranNumofTroops,1,-1
        do
          local group = CSAR.CSAREnemyTroopSpawn:SpawnFromVec2( SpawnVec3:GetVec2() )  
          EnemyGroupSet:AddGroupsByName(group)
        end
        
        local ranNumofBTRs = math.random(1,2) --gen random number of troops math.random(3,5)
        for EnemyBTRCount = ranNumofBTRs,1,-1
        do
          local group = CSAR.CSAREnemyBTR:SpawnFromVec2( SpawnVec3:GetVec2() )
          EnemyGroupSet:AddGroupsByName(group)
        end
        
      --Create Beacon
      local beaconID = TheFrat.CreateBeacon(Zone,RadioFreq)
            
       --Set Briefing Menu
       local rescueType = CSAR.randomchoice(CSAR.AirplaneType)
       local rescueRank = CSAR.randomchoice(CSAR.Ranks)
       local rescureName = CSAR.randomchoice(CSAR.Names)
       local Briefing = "Mission # "..CSAR_Task_ID.."\n".."Rescue Downed "..rescueType.." Pilot: "..rescueRank.." "..rescureName.."\n"..
       CSAR.GetBearingToObj(PlayerGroupName,CSARTroopGroup:GetName()).."\nRadio Freq: ADF "..RadioFreq.." MHZ FM"
       
       CSAR.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
       MESSAGE:New(Briefing,35):ToGroup(PlayerGroup)
       
       local function showBriefing()
       
        CSAR.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
        MESSAGE:New(Briefing,35):ToGroup(PlayerGroup)
      end
      local BriefingMenu = MENU_GROUP:New( PlayerGroup, "Your Assignment" )
      local BriefingMenuCommand = MENU_GROUP_COMMAND:New( PlayerGroup, "Briefing", BriefingMenu, showBriefing , PlayerGroup ) 
      --BriefingMenuCommand:Remove()
      --BriefingMenu:Remove()
      
      --CREATE CSAR Task in DB
      local lz_lat, lz_long = coord.LOtoLL(Zone:GetPointVec3())
      --EVENTS
      --Takeoff
      PlayerGroup:HandleEvent(EVENTS.Takeoff)
      function PlayerGroup:OnEventTakeoff(EventData)
        
        if firstTakeoff == false then
          firstTakeoff = true
          --Leaving Safe Zone Check
          TheFrat.LeavingZoneCheck(PlayerGroup, Settings.SafeZone,"FARP Hollywood")
        end
      end--end EVENTS.Takeoff
      
      --Landing
      PlayerGroup:HandleEvent(EVENTS.Land)
        function PlayerGroup:OnEventLand(EventData)
        local PlayerUnit = EventData.IniUnit
        local PlayerName = PlayerUnit:GetPlayerName()
          --check if picking up downed pilot
          if PlayerGroup:IsCompletelyInZone(CSARZone) and troopPickedUp == false then --SET Step 06
             SCHEDULER:New( nil,
              function()
                 --load troops
                  --InfantryCargo:Board(PlayerGroup,25)  
                  InfantryCargo:Board(PlayerGroup,22) 
                  troopPickedUp = true
                 
                  CSAR.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
                  PlayerGroup:Message("= Troop now boarding =",5)
                  CSARFlare:Stop()
                  TheFrat.StopBeacon(beaconID,RadioFreq) 
                  
                  --check if in home airspace, if so flare
                  SCHEDULER:New( nil,
                  function()
                    if PlayerGroup:IsCompletelyInZone(ZONE:New("Hollywood Airspace")) and inAirSpace == false then
                      inAirSpace = true
                      TheFrat.FARPHollywoodSignal()
                    end
                  end, {},300,15,0,3600)

              end, {}, 2)
          end
          
          --If Landing in Safe Zone and troop nounload then unload troopUnloaded=false
          if CSAR.CheckIfInSafeZone(PlayerGroup,AlliedLandingZones) == true and troopPickedUp == true and troopUnloaded == false then --SET Step 06
             SCHEDULER:New( nil,
              function()
                 --load troops
                 --MESSAGE:New("Unload...",3):ToAll()
                 --InfantryCargo:UnBoard(POINT_VEC2:NewFromVec2(ZONE:New(TroopHoldZoneName):GetRandomVec2()))
                 InfantryCargo:UnBoard()
                 troopUnloaded = true
                 
                 function InfantryCargo:onafterUnBoarding(ToPointVec2,Event,From,To,NearRadius)
                    --MESSAGE:New("On After Unboard Fire...",3):ToAll()
                    
                    BriefingMenu:Remove()
                    CSAR.SetMooseClientStateData(PlayerGroup,"hasAssignment", false)
                    CSARCompleted = true
                    
                    --Task Successful
                    
                    local missionMarkerText = "Mission ID: "..CSAR_Task_ID.." - CSAR\n"..
                                              "Pilot: "..PlayerName.."\n"..
                                              rescueRank..""..rescureName.." rescued!"
                    TheFrat.CreateMarker(Zone,missionMarkerText,true)
                    
                    local MarkerText = "Mission ID: "..CSAR_Task_ID.." - CSAR\\n"..
                                              "Pilot: "..PlayerName.."\\n"..
                                              rescueRank..""..rescureName.." rescued!"
                    --TheFrat.AddMarkerTable(Zone,CSAR_Task_ID,MarkerText,TheFrat.GetMooseClientStateData(PlayerGroup,"ucid"))
                                        
                    --Acknowledge
                    SCHEDULER:New( nil,
                      function()
                        TheFrat.PlaySoundTAll("info.ogg")
                        MESSAGE:New("CSAR Mission ID: "..CSAR_Task_ID.."\nMission Successful!\n"..rescueRank..""..rescureName.." has been rescued! Good Work "..PlayerName.."!",20,"HQ"):ToAll()
                        

                          --Remove Rescued Solider
                           SCHEDULER:New( nil,
                              function()
                                InfantryCargo:Destroy()
                              end, {},30)
                          
                      end, {},4)

                    
                 end
                 
              end, {}, 2)
          end
          
      end--EVENTS.Land
      
      --PlayerLeaveUnit
      PlayerGroup:HandleEvent(EVENTS.PlayerLeaveUnit)
      function PlayerGroup:OnEventPlayerLeaveUnit(EventData)
        BriefingMenu:Remove()
        CSAR.DestroyGroupSet(EnemyGroupSet)
        CSAR.SetMooseClientStateData(PlayerGroup,"hasAssignment", false)

        if InfantryCargo:IsAlive() ~= nil and InfantryCargo:IsAlive() == true then
          InfantryCargo:Destroy()
        end
        TheFrat.StopBeacon(beaconID,RadioFreq)

      end--end EVENTS.PlayerLeaveUnit

      --Player DEAD
      PlayerGroup:HandleEvent(EVENTS.Dead)
      function PlayerGroup:OnEventDead(EventData)
        BriefingMenu:Remove()
        CSAR.DestroyGroupSet(EnemyGroupSet)
        CSAR.SetMooseClientStateData(PlayerGroup,"hasAssignment", false)

        if InfantryCargo:IsAlive() ~= nil and InfantryCargo:IsAlive() == true then
          InfantryCargo:Destroy()
        end
        TheFrat.StopBeacon(beaconID,RadioFreq)

      end
      
      --Player Crash
      PlayerGroup:HandleEvent(EVENTS.Crash)
      function PlayerGroup:OnEventCrash(EventData)
        BriefingMenu:Remove()
        CSAR.DestroyGroupSet(EnemyGroupSet)
        CSAR.SetMooseClientStateData(PlayerGroup,"hasAssignment", false)

        if InfantryCargo:IsAlive() ~= nil and InfantryCargo:IsAlive() == true then
          InfantryCargo:Destroy()
        end
        TheFrat.StopBeacon(beaconID,RadioFreq)

      end
      
      
    else
      --no zones avalible
      CSAR.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
      MESSAGE:New("Currently no CSAR Assingments at this time, please try back later.",10):ToGroup(PlayerGroup)
    end
  
  end

end

--Make CSAR Menu Avalible
function CSAR:Menu(playerGroup)
  local PlayerGroup = playerGroup

  --Setup Menu Option
  local function ShowInst()
      local inst = "++ CSAR Instructions ++"
      CSAR.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
      MESSAGE:New(inst,10):ToGroup(PlayerGroup)
    end
  local function GetAssignment()
    CSAR.PlaySoundToGroup("beep1.ogg",PlayerGroup)
    MESSAGE:New(">> Fetching CSAR Assigment for "..PlayerGroup:GetName(),5):ToGroup(PlayerGroup)
    SCHEDULER:New( nil,
      function()
        CSAR.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
        CSAR:New(PlayerGroup,CSAR.Settings.CSARZonesTable,CSAR.Settings.AlliedLandingZones,CSAR.Settings.TroopHoldZoneName)
      end, {},6)
    
  end
  
  if CSAR.ClientInTable(CSAR.Settings.clientTransportsTable,PlayerGroup:GetName()) then
    local CSARMenu = MENU_GROUP:New( PlayerGroup, "Troop Transport" )
    --local CSARInstrustions = MENU_GROUP_COMMAND:New( PlayerGroup, "Instructions", CSARMenu, ShowInst , PlayerGroup ) 
    local GetCSARAssignment = MENU_GROUP_COMMAND:New( PlayerGroup, "Get CSAR Assignment", CSARMenu, GetAssignment , PlayerGroup )
  end
 
end

function CSAR:GetCSARUnit(UnitList)
  local CSARTroopUnit
  local CSARTroopUnitName
  
  for i, Unit in ipairs(UnitList) do
      CSARTroopUnitName = Unit:GetName() --Set CSAR Unit Name
      CSARTroopUnit = Unit --Set CSAR Unit
   end
   return CSARTroopUnit
end

do--CSAR Helpers

CSAR.AirplaneType = {
  "KA-50",
  "F-16",
  "F/A 18",
  "A-10",
  "F5-E",
}

CSAR.Ranks = {
  "Sergeant",
  "Captain",
  "Sergeant First Class",
  "Lieutenant",
  "Major",
  "First Lieutenant",
  "Staff Sergeant",
}

CSAR.Names = {
  "Morgan",
  "Cooper",
  "Phillips",
  "Sanders",
  "Patel",
  "Taylor",
  "Kindrick",
  "Ballard",
  "Anderson",
  "Luther",
  "Riddle",
  "Gardner",
  "Michell",
  "Cole",
  "Strong",
  "Fisher",
  "Sully",
  "Wright",
  "Tilly",
}

CSAR.Squads = {
  "Jay Squad",
  "Blue Ray Squad",
  "The Wolf Pack Squad",
  "the Red Squad",
  "The Black Head Squad",
}

CSAR.FreqWattage = 10 --Watts to broadcast radio

end

function CSAR.GetBearingToObj(from,to) 
  local group_1_pos = Group.getByName(from):getUnits()[1]:getPoint()
  local group_2_pos = Group.getByName(to):getUnits()[1]:getPoint()
  local distance = ((group_1_pos.x - group_2_pos.x)^2 + (group_1_pos.z - group_2_pos.z)^2)^0.5
  local bearing_vector = {
    x = group_2_pos.x - group_1_pos.x, 
    y = group_2_pos.y - group_1_pos.y, 
    z = group_2_pos.z - group_1_pos.z
    }
  local bearing_rad = math.atan2(bearing_vector.z, bearing_vector.x)
  if bearing_rad < 0 then
      bearing_rad = bearing_rad + (2 * math.pi) 
    end
    local bearing = math.deg(bearing_rad)
    return 'Go Bearing '..math.floor(bearing)..' Degrees for about '..math.floor(UTILS.MetersToNM(distance)).." Nm."
end

function CSAR.RandomVariable(length)
  local res = ""
  for i = 1, length do
    res = res .. string.char(math.random(97, 122))
  end
  return res
end

function CSAR.CheckIfInSafeZone(PlayerGroup,safeLandingZones)
  local result = false
  for i = 1, #safeLandingZones do
    --print(i, ". ", safeLandingZones[i])
      if PlayerGroup:IsCompletelyInZone(safeLandingZones[i]) then
        result = true
      end
  end
  return result
end

function CSAR.ClientInTable (tab, val)
  for index, value in ipairs(tab) do
    if value == val then
      return true
    end
  end

  return false
end

function CSAR.CountGroupSet(groupSet)
local count = 0
groupSet:ForEachGroup(function(group)
        if group:IsAlive() then
          count = count + 1
        end
      end)
      return count
end

--play SoundFile to User Group
function CSAR.PlaySoundToGroup(soundfile,group)
  local PlayAudioFile = USERSOUND:New( soundfile )
  PlayAudioFile:ToGroup(group)
end

--play SoundFile to All Users
function CSAR.PlaySoundTAll(soundfile)
  local PlayToAll = USERSOUND:New( soundfile )
  PlayToAll:ToAll()
end

-- Select random item from table
function CSAR.randomchoice(t) --Selects a random item from a table
  local keys = {}
  for key, value in pairs(t) do
    keys[#keys+1] = key --Store keys in another table
  end
  index = keys[math.random(1, #keys)]
  return t[index]
end
function CSAR.sanitise( str )
    str = str:gsub('%W','')
    return str
end

function CSAR.tablefind(tab,el)
    for index, value in pairs(tab) do
        if value == el then
            return index
        end
    end
end

function CSAR.removeTableEntry(tabel,val)
  table.remove( tabel, CSAR.tablefind(tabel, val) )
end

--Count number of items within table
function CSAR.tablelength(T)
  local count = 0
  for _ in pairs(T) do count = count + 1 end
  return count
end

--Explode all units within a group
function CSAR.explode(group)
  local unitsList = group:GetUnits()
    for i, unit in ipairs(unitsList) do
      CSAR.explodeUnit(unit)
    end
end

function CSAR.DestroyGroupSet(groupSet)--Detroy (remove) every unit in group set
  groupSet:ForEachGroup(function(group)
  
  if group:IsAlive() ~= nil and group:IsAlive() == true then
    group:Destroy()
   end
  end)
end

--Explode all units in group set. Requires a SET Group
function CSAR.explodeGroupSet(groupSet)
  groupSet:ForEachGroup(function(group)
    if group:IsAlive() ~= nil and group:IsAlive() == true then
      CSAR.explode(group)
    end
  end)
  
end

--Explode Unit
function CSAR.explodeUnit(unit)
  local boom1pos = unit:GetPointVec2()
  local pos = POINT_VEC2:New( boom1pos:GetLat(), boom1pos:GetLon() )
  trigger.action.explosion(pos, 100)
end

--Explode Static
function CSAR.explodeStatic(static)
  local boom1pos = static:GetPointVec2()
  local pos = POINT_VEC2:New( boom1pos:GetLat(), boom1pos:GetLon() )
  trigger.action.explosion(pos, 100)
end

function CSAR.SetMooseClientStateData(PlayerGroup,SetData, Data)
  local unitName = CSAR.GetUnitFromPlayer(PlayerGroup:GetPlayerUnits())
  local Client = SET_CLIENT:New():AddClientsByName(unitName)
    Client:ForEachClient(
      function(MooseClient)
        MooseClient:SetState( MooseClient, SetData, Data )
      end
    )
  Client = nil
end

function CSAR.GetMooseClientStateData(PlayerGroup,StateData)
  local data = ""
  local unitName = CSAR.GetUnitFromPlayer(PlayerGroup:GetPlayerUnits())
  local Client = SET_CLIENT:New():AddClientsByName(unitName)
    Client:ForEachClient(
      function(MooseClient)
        --MESSAGE:New('Your UCID '..MooseClient:GetState( MooseClient, "UCID" ),10):ToClient(MooseClient)
        data = MooseClient:GetState( MooseClient, StateData )
      end
    )
  Client = nil
  return data
end
function CSAR.GetUnitFromPlayer(GetPlayerUnitsTable)
  local unitsTables = GetPlayerUnitsTable
  for UnitId, UnitData in pairs( unitsTables ) do
      local UnitAction = UnitData -- Wrapper.Unit#UNIT
      return UnitAction:Name()
    end
end

function CSAR.BroadcastRadioBeacon(group,radioFreq)
  local Frequency = radioFreq * 1000000 -- Conversion to Hz
  local audio = "l10n/DEFAULT/".."beacon.ogg"
  trigger.action.radioTransmission(audio, Group.getByName(group:GetName()):getUnit(1):getPoint(), radio.modulation.FM, false,Frequency, 1000)
end

return CSAR