Logistics = Class:derive("Logistics")
Logistics.Settings = {}

--Settings
Logistics.Settings.MaxNumberOfOps = 5

--List all pickup and drop off zones
Logistics.Settings.LogisticZones = {
  ZONE:New("FARP Hollywood Logistics"),
  ZONE:New("FARP Dallas Logistics"),
  ZONE:New("Muhuri Supply Depot"),
  ZONE:New("Zugdidi Ammunitions Depot"),

}
Logistics.Settings.CurrentNumberOfOps = 0
function Logistics:New(playerGroup)
  local PlayerGroup = playerGroup
  
  --check if already assigned a transport Assignment
  if TheFrat.GetMooseClientStateData(PlayerGroup,"hasAssignment") == true then
    TheFrat.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
    MESSAGE:New("You have already been assigned a Task. Please review your briefing from F10 Menu. Complete you current task before selecting a new one.",15):ToGroup(PlayerGroup)
  else
    
    if Logistics.Settings.CurrentNumberOfOps <= Logistics.Settings.MaxNumberOfOps then
      
      --set some vars
      local firstTakeoff = false
      local missionCompleted = false
      local inPickUpAirSpace = false
      local inPickDopAirSpace = false
      local packagePickedUp = false
      local packageDroppedOff = false
      local Task_ID = TheFrat.RandomNumber()
      local PickUpZone = TheFrat.randomchoice(Logistics.Settings.LogisticZones)
      local DropOffZone = Logistics.GetRandomDropOffZone(PickUpZone)
      local LogisticSpawn = TheFrat.randomchoice(Logistics.SpawnCrateObject)
      local logisticItemName = "logistic_item_"..TheFrat.RandomVariable(4)
      LogisticSpawn:SpawnFromPointVec2(PickUpZone:GetRandomPointVec2(),0,logisticItemName)
      local StaticTransport = STATIC:FindByName(logisticItemName) --STATIC object
      local StaticType = StaticTransport:GetTypeName() --Type of cargo being transported
      local LZ_AirSpace = ZONE_RADIUS:New( "logistical_airspace_"..TheFrat.RandomVariable(3), StaticTransport:GetVec2(), 3000 )
      local DropZoneAirSpace = ZONE_RADIUS:New( "logistical_drop_airspace_"..TheFrat.RandomVariable(3), DropOffZone:GetVec2(), 3000 )
      local LogisticCargo = CARGO_CRATE:New(STATIC:FindByName(logisticItemName) ,logisticItemName,logisticItemName)

      --local testStaticCargo = CARGO_CRATE:New(testStatic,"test","test"):Load(PlayerGroup)
      
      --moose client hasAssigment to true
      TheFrat.SetMooseClientStateData(PlayerGroup,"hasAssignment", true)
      
      --Spawn Random Enemy Units in route to LZ when not in allied zone
      local noEnemyThreatZones = Settings.NoThreatsZones --table
      
      --pop flare/Smoke when near
      if StaticTransport:GetCoordinate():IsDay() == true then
        --pop smoke
        local Smoke = SCHEDULER:New( nil,
          function()
            if PlayerGroup:IsCompletelyInZone(LZ_AirSpace) and inPickUpAirSpace == false and PlayerGroup:IsCompletelyInZone(ZONE:New('FARPHollywood')) == false then
            inPickUpAirSpace = true
            TheFrat.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
            PlayerGroup:Message("Poping Green Smoke at your Pick up Point.",15,"Operator")
            --pop smoke
            StaticTransport:SmokeGreen()
            
          end
        end, {}, 1,1,0,5400)
      else 
        --pop flare
        local Flare = SCHEDULER:New( nil,
          function()
            if PlayerGroup:IsCompletelyInZone(LZ_AirSpace) and inPickUpAirSpace == false and PlayerGroup:IsCompletelyInZone(ZONE:New('FARPHollywood')) == false then
            inPickUpAirSpace = true
            TheFrat.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
            PlayerGroup:Message("Poping Green Flare at your Pick up Point.",15,"Operator")
            --pop flare
          SCHEDULER:New( nil,
            function()
              StaticTransport:FlareGreen()
            end, {},2,4,0,120)
          end
        end, {}, 1,1,0,5400)
      end
       
    
      --Set Briefing Menu
      local brief = "Mission #: "..Task_ID.."\n"..
          "Pilot : "..PlayerGroup:GetPlayerName().."\n"..
          "Logistical Objective : Transport "..StaticType.."\n"..
          "Pick Up Location :"..PickUpZone:GetName().."\n"..
          "Drop Off Location :"..DropOffZone:GetName().."\n"..
          "______________________\n\n".. 
          "Mission : You have been assigned a logistical objective of transporting "..StaticType.." to "..DropOffZone:GetName()..". Your package has been deployed and is waiting for your arrival.\n"..
          "______________________\n\n".. 
          "To Pick Up Zone: "..TheFrat.GetBearingToZone(PlayerGroup:GetVec3(),PickUpZone:GetVec3())..'\n'..
          "Pick up Zone to Drop Zone: "..TheFrat.GetBearingToZone(PickUpZone:GetVec3(),DropOffZone:GetVec3())..'\n'..
          "Check briefing for Warehous/Supply Depot ADF Freqs"
           
      TheFrat.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
      MESSAGE:New(brief,45):ToGroup(PlayerGroup)
       
      local function showBriefing()
        TheFrat.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
        MESSAGE:New(brief,35):ToGroup(PlayerGroup)
      end
      local BriefingMenu = MENU_GROUP:New( PlayerGroup, "Your Assignment" )
      local BriefingMenuCommand = MENU_GROUP_COMMAND:New( PlayerGroup, "Briefing", BriefingMenu, showBriefing , PlayerGroup )
      
      --CREATE Task in DB
      local lz_lat, lz_long = coord.LOtoLL(PickUpZone:GetPointVec3())
     
      --EVENTS
      --Takeoff
      PlayerGroup:HandleEvent(EVENTS.Takeoff)
      function PlayerGroup:OnEventTakeoff(EventData)
        
        if firstTakeoff == false then
          firstTakeoff = true
          --Leaving Safe Zone Check
          TheFrat.LeavingZoneCheck(PlayerGroup, Settings.SafeZone,"FARP Hollywood")
        end
      end--end EVENTS.Takeoff
      
      --Landing
      PlayerGroup:HandleEvent(EVENTS.Land)
      function PlayerGroup:OnEventLand(EventData)
        --UNIT:FindByName(UnitName)
        local TargetUnit = EventData.IniUnit
        
        --check if in pickup zone
        if PlayerGroup:IsCompletelyInZone(PickUpZone) and packagePickedUp == false then
          SCHEDULER:New( nil,
            function()
              LogisticCargo:Load(PlayerGroup)
              Flare:Stop()
              packagePickedUp = true
              TheFrat.PlaySoundTAll("radioscratch.ogg")
              MESSAGE:New(TargetUnit:GetPlayerName().." has loaded "..StaticType.." into his "..TargetUnit:GetTypeName().." for transport.",8):ToAll()
              local zoneFlare = ZONE_RADIUS:New( "zone_flare_"..TheFrat.RandomVariable(3), DropOffZone:GetVec2(), 4 )
              --pop flare when near Drop Off Zone 
               local DropFlare = SCHEDULER:New( nil,
                function()
                  if PlayerGroup:IsCompletelyInZone(DropZoneAirSpace) and inPickDopAirSpace == false then
                  inPickDopAirSpace = true
                  TheFrat.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
                  PlayerGroup:Message("Poping Green Flare at your Drop Off Area.",15,"Operator")
                  --pop flare
                SCHEDULER:New( nil,
                  function()
                    zoneFlare:FlareZone(FLARECOLOR.Green,1)
                  end, {},2,4,0,180)
                end
              end, {}, 1,1,0,5400)
              
            end, {},3)
        end
        
        --check if in drop off zone
        if PlayerGroup:IsCompletelyInZone(DropOffZone) and packageDroppedOff == false then
          SCHEDULER:New( nil,
            function()
              LogisticCargo:UnLoad()
              packageDroppedOff = true
              --moose client hasAssigment to true
              TheFrat.SetMooseClientStateData(PlayerGroup,"hasAssignment", false)
              TheFrat.PlaySoundTAll("radioscratch.ogg")
              missionCompleted = true
              MESSAGE:New(TargetUnit:GetPlayerName().." has delivered "..StaticType.." to "..DropOffZone:GetName().." in his "..TargetUnit:GetTypeName()..". Good Work!",16,"Logistical Update"):ToAll()
              Logistics.Settings.CurrentNumberOfOps = Logistics.Settings.CurrentNumberOfOps + 1
              --Task Successful
              --Destroy Cargo
              SCHEDULER:New( nil,
              function()
                StaticTransport:Destroy()
              end, {},30)
              BriefingMenu:Remove()
              
            end, {},3)
        end
      
      end
      
      --Leave Unit
      PlayerGroup:HandleEvent(EVENTS.PlayerLeaveUnit)
      function PlayerGroup:OnEventPlayerLeaveUnit(EventData)
        if missionCompleted == false then
           StaticTransport:Destroy()
           BriefingMenu:Remove()
           TheFrat.SetMooseClientStateData(PlayerGroup,"hasAssignment", false)

        end
        
      end
      
      --Pilot Dead
      PlayerGroup:HandleEvent(EVENTS.Dead)
      function PlayerGroup:OnEventDead(EventData)
        if missionCompleted == false then
           StaticTransport:Destroy()
           BriefingMenu:Remove()
           TheFrat.SetMooseClientStateData(PlayerGroup,"hasAssignment", false)
        end
        
      end
      
      --Pilot Crash
      PlayerGroup:HandleEvent(EVENTS.Crash)
      function PlayerGroup:OnEventCrash(EventData)
        if missionCompleted == false then
           StaticTransport:Destroy()
           BriefingMenu:Remove()
           TheFrat.SetMooseClientStateData(PlayerGroup,"hasAssignment", false)
        end
        
      end
      
      

    else
      --no logistics ops avalible
      TheFrat.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
      MESSAGE:New("No logistical operations at this time. Please try back later.",15):ToGroup(PlayerGroup)
    end
    
  end
  
end

function Logistics:Menu(playerGroup)
  local PlayerGroup = playerGroup
  local PlayerGroupName = PlayerGroup:GetName()
  --Setup Menu Option
  local function ShowInst()
      local inst = "++ Logistics Instructions ++"
      TheFrat.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
      MESSAGE:New(inst,10):ToGroup(PlayerGroup)
      
    end
  local function GetAssignment()
    TheFrat.PlaySoundToGroup("beep1.ogg",PlayerGroup)
    MESSAGE:New(">> Fetching Logistics Assigment for "..PlayerGroupName,5):ToGroup(PlayerGroup)
    SCHEDULER:New( nil,
      function()
        TheFrat.PlaySoundToGroup("radioscratch.ogg",PlayerGroup)
        Logistics:New(PlayerGroup)
      end, {},6)
    
  end
  
  if TheFrat.ClientInTable(Settings.clientTransportsTable,PlayerGroupName) then
    local logisticsmenu = MENU_GROUP:New( PlayerGroup, "Troop Transport" )
    --local LogInstrustions = MENU_GROUP_COMMAND:New( PlayerGroup, "Instructions", LogisticsMenu, ShowInst , PlayerGroup )--instr in CSAR class
    local GetLogssignment = MENU_GROUP_COMMAND:New( PlayerGroup, "Get Logistics Assignment", logisticsmenu, GetAssignment , PlayerGroup )
  end
end

function Logistics.GetRandomDropOffZone(PickUpZone)
  
  local ranChoice = TheFrat.randomchoice(Logistics.Settings.LogisticZones)
  if ranChoice ~= PickUpZone then
    return ranChoice
  else
    return Logistics.GetRandomDropOffZone(PickUpZone)
  end
  
end

Logistics.SpawnCrateObject = {
  SPAWNSTATIC:NewFromStatic( "Ammo Crate", country.id.USA ),
  SPAWNSTATIC:NewFromStatic( "Fuel Barrels", country.id.USA ),
  SPAWNSTATIC:NewFromStatic( "Container", country.id.USA ),
  SPAWNSTATIC:NewFromStatic( "Fueltank", country.id.USA ),
  SPAWNSTATIC:NewFromStatic( "Oiltank", country.id.USA ),
}
